package com.hzya.frame.plugin.cbs8.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.cbs8.dto.req.*;
import com.hzya.frame.cbs8.dto.res.*;
import com.hzya.frame.cbs8.service.ICbs8Service;
import com.hzya.frame.cbs8.util.CBSUtil;
import com.hzya.frame.cbs8.util.PayState;
import com.hzya.frame.plugin.cbs8.service.ICbsPluginService;
import com.hzya.frame.seeyon.cap4.form.dto.*;
import com.hzya.frame.seeyon.cbs8.entity.AgentPaymentDetailEntity;
import com.hzya.frame.seeyon.cbs8.entity.AgentPaymentEntity;
import com.hzya.frame.seeyon.cbs8.entity.CbsLogEntity;
import com.hzya.frame.seeyon.cbs8.entity.PaymentEntity;
import com.hzya.frame.seeyon.cbs8.service.ICbsLogService;
import com.hzya.frame.seeyon.cbs8.service.IPaymentService;
import com.hzya.frame.seeyon.entity.CtpAttachmentEntity;
import com.hzya.frame.seeyon.entity.CtpFileEntity;
import com.hzya.frame.seeyon.service.ICtpAttachmentService;
import com.hzya.frame.seeyon.util.RestUtil;
import com.hzya.frame.uuid.UUIDLong;
import com.hzya.frame.web.exception.BaseSystemException;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @Description
 * @Author xiangerlin
 * @Date 2024/6/17 08:46
 **/
public class CbsPluginServiceImpl implements ICbsPluginService {

    Logger logger = LoggerFactory.getLogger(CbsPluginServiceImpl.class);
    @Autowired
    private ICbs8Service cbs8Service;
    @Autowired
    private IPaymentService paymentService;
    @Autowired
    private ICbsLogService cbsLogService;
    @Autowired
    private ICtpAttachmentService ctpAttachmentService;

    @Autowired
    private RestUtil restUtil;
    @Value("${cbs8.elec_path}")
    private String elec_path;
    @Value("${OA.data_source_code}")
    private String oa_data_source_code;
    /**
     * 支付申请
     *
     * @param paymentEntity
     */
    @Override
    public void applyPay(PaymentEntity paymentEntity) throws Exception{
        //查询待支付的列表
        if (null == paymentEntity){
            paymentEntity = new PaymentEntity();
        }
        paymentEntity.setDataSourceCode(oa_data_source_code);
        List<PaymentEntity> paymentList = paymentService.queryUnpaid(paymentEntity);
       /* List<PaymentEntity> paymentList = new ArrayList<>();
        paymentEntity.setReferenceNum("CL202406140001");
        paymentEntity.setPayAccount("655905707410000");
        paymentEntity.setPayBankName("");
        paymentEntity.setAmount("99");
        paymentEntity.setRevAccount("123456778");
        paymentEntity.setRevBankName("中国工商银行总行清算中心");
        paymentEntity.setRevBankType("ICB");
        paymentEntity.setRevAccountName("测试账户");
        paymentEntity.setCnapsCode("102100099996");
        paymentEntity.setPurpose("测试用途");
        paymentEntity.setBusType("202");
        paymentEntity.setCurrency("10");
        paymentEntity.setPurpose("测试用途");
        paymentList.add(paymentEntity);*/
        if (CollectionUtils.isNotEmpty(paymentList)){
            for (PaymentEntity pay : paymentList) {
                //调用支付申请接口
                PayResponseDTO payResponseDTO = cbs8Service.payApply(pay);
                boolean successed = payResponseDTO.getSuccessed();
                if (successed){
                    pay.setPayResult(PayState.p.getValue());
                }else {
                    pay.setPayResult("推送失败");
                }
                //4、更新OA表单
                pay.setDataSourceCode(oa_data_source_code);
                paymentService.updatePayState(pay);
                //5、记录操作日志
                cbsLogService.saveLog(new CbsLogEntity());
            }
        }
    }

    /**
     * 查询支付申请的交易结果
     *
     * @param cbsLogEntity
     * @throws Exception
     */
    @Override
    public void queryResult(CbsLogEntity cbsLogEntity) throws Exception {
        if (null == cbsLogEntity){
            cbsLogEntity = new CbsLogEntity();
        }
        // 1、查询支付中的日志
        List<CbsLogEntity> inPayList = cbsLogService.queryInPayment(cbsLogEntity);
        if (CollectionUtils.isNotEmpty(inPayList)){
            for (CbsLogEntity entity : inPayList) {
                try {
                    List<PayResultResDTO> payResultResList = cbs8Service.queryPayResult(new PayResultRequestDTO(entity.getBill_code()));
                    if (CollectionUtils.isNotEmpty(payResultResList)){
                        PayResultResDTO payResultResDTO = payResultResList.get(0);
                        //支付申请状态
                        String status = payResultResDTO.getStatus();
                        //支付状态
                        String pay_status = payResultResDTO.getPayStatus();
                        //不等于支付中的时候 更新支付状态
                        if (!PayState.p.getType().equals(pay_status)){
                            //如果支付状态为空，保存支付申请状态，如果支付状态不为空，则保存支付状态
                            PaymentEntity paymentEntity = new PaymentEntity();
                            paymentEntity.setOaId(entity.getOa_id());
                            List<PaymentEntity> paymentList = paymentService.query(paymentEntity);
                            if (CollectionUtils.isNotEmpty(paymentList)){
                                paymentEntity = paymentList.get(0);
                                if (StrUtil.isEmpty(pay_status)) {
                                    //支付申请状态 支付状态和支付申请状态用一个
                                    paymentEntity.setPayResult(PayState.payStateGetValue(status));
                                } else {
                                    //支付状态 支付状态和支付申请状态用一个
                                    paymentEntity.setPayResult(PayState.payStateGetValue(pay_status));
                                }
                                if (StrUtil.isNotEmpty(pay_status) && pay_status.equals(PayState.g.getType())) {
                                    //支付时间
                                    paymentEntity.setPayDate(CBSUtil.convertTimestampToString(payResultResDTO.getPayDate()));
                                }
                                //更新视图单据状态
                                paymentService.updatePayState(paymentEntity);
                                //更新日志表状态
                                entity.setPay_state(paymentEntity.getPayResult());
                                entity.setApply_state(PayState.payStateGetValue(status));
                                cbsLogService.update(entity);
                            }
                        }
                    }
                }catch (Exception e){
                 e.printStackTrace();
                 logger.error("查询交易结果出错",e);
                }
            }
        }
    }

    /**
     * 电子回单查询 并上传OA
     *
     * @param requestJson
     * @throws Exception
     */
    @Override
    public void elecBillUpload(JSONObject requestJson) throws Exception {
        //查询支付成功 没有电子回单的数据
        PaymentEntity paymentEntity = new PaymentEntity();
       // List<PaymentEntity> paymentList = paymentService.queryElecIsNull(paymentEntity);
        paymentEntity.setPayDate("2024-06-20");
        paymentEntity.setReferenceNum("41");
        List<PaymentEntity> paymentList = Arrays.asList(paymentEntity);
        if (CollectionUtils.isNotEmpty(paymentList)) {
            for (PaymentEntity pay : paymentList) {
                try {
                    String payDate = DateUtil.format(DateUtil.parse(pay.getPayDate()), "yyyy-MM-dd");
                    //查询cbs电子回单
                    List<ElecResponseDTO> elecResList = cbs8Service.queryElecBill(new ElecRequestDTO(payDate,DateUtil.today(),pay.getReferenceNum()));
                    if (CollectionUtils.isNotEmpty(elecResList)){
                        ElecResponseDTO elecResponseDTO = elecResList.get(0);
                        String bucketFileUrl = elecResponseDTO.getBucketFileUrl();
                        String bucketFileName = elecResponseDTO.getBucketFileName();
                        //上传电子回单到OA
                        HttpUtil.downloadFile(bucketFileUrl, FileUtil.file(elec_path));//附件下载
                        String pdfUrl = elec_path + bucketFileName;
                        File file = new File(pdfUrl);
                        if (file.exists()) {
                            CtpFileEntity cpFileEntity =  new CtpFileEntity();
                            cpFileEntity.setFile(file);
                            cpFileEntity.setDataSourceCode(oa_data_source_code);
                            JSONObject jsonObjectUpload = restUtil.fileUpload(file,"8000240005");
                            String file_url = jsonObjectUpload.getString("fileUrl");
                            if (null != jsonObjectUpload && StrUtil.isNotEmpty(file_url)) {
                                String sub_reference = String.valueOf(UUIDLong.longUUID());
                                pay.setReceipt(sub_reference);
                                //更新表单的电子回单值
                                paymentService.updateElec(pay);
                                //保存附件关系
                                CtpAttachmentEntity ctpAttachmentEntity = new CtpAttachmentEntity();
                                ctpAttachmentEntity.setFile_url(file_url);
                                ctpAttachmentService.saveAttachment(file_url, pay.getSummaryId(), sub_reference);
                            }
                            //删除本地临时文件
                            file.delete();
                        }
                    }
                }catch (Exception e){
                    logger.error("电子回单查询出错",e);
                }
            }
        }
    }

    /**
     * 查询交易明细
     * transactionDetailReqDTO.currentPage 、 transactionDetailReqDTO.pageSize 必填
     * @param transactionDetailReqDTO
     */
    @Override
    public List<TransactionDetailDTO> queryTransactionDetail(TransactionDetailReqDTO transactionDetailReqDTO) {
        boolean hasNextPage = true;//是否有下一页
        int currentPage = transactionDetailReqDTO.getCurrentPage();
        int pageSize = transactionDetailReqDTO.getPageSize();
        if (currentPage == 0){
            currentPage = CBSUtil.DEFAULT_CURRENT_PAGE;//页码
        }
        if (pageSize == 0){
            pageSize = CBSUtil.DEFAULT_PAGE_SIZE;//每页条数
            transactionDetailReqDTO.setPageSize(pageSize);
        }
        List<TransactionDetailDTO> resultList = new ArrayList<>();
        do{
            transactionDetailReqDTO.setCurrentPage(currentPage);//页码
            currentPage++;//页码自增
            hasNextPage = false;//保护功能，防止出现死循环
            CbsResDataDTO dataDTO = cbs8Service.queryTransactionDetail(transactionDetailReqDTO);
            if (null != dataDTO){
                hasNextPage = dataDTO.getHasNextPage();
                List<TransactionDetailDTO> transactionDetailDTOList = CBSUtil.convertJsonArrayToList(dataDTO.getList(), TransactionDetailDTO.class);
                resultList.addAll(transactionDetailDTOList);
            }
        }while (hasNextPage);

        return resultList;
    }

    /**
     * 代发代扣 支付申请
     *
     * @param paymentEntity
     * @param agentPaymentDetailList
     */
    @Override
    public PayResponseDTO applyAgentPay(AgentPaymentEntity paymentEntity, List<AgentPaymentDetailEntity> agentPaymentDetailList) {
        if (null != paymentEntity && CollectionUtils.isNotEmpty(agentPaymentDetailList)){
            PaymentApplySubmitReqDTO paymentApplySubmitReqDTO = BeanUtil.copyProperties(paymentEntity,PaymentApplySubmitReqDTO.class);
            List<PaymentApplyAgentDTO> paymentApplyAgentList = new ArrayList<>();
            for (AgentPaymentDetailEntity detailEntity : agentPaymentDetailList) {
                PaymentApplyAgentDTO detailDTO = BeanUtil.copyProperties(detailEntity,PaymentApplyAgentDTO.class);
                paymentApplyAgentList.add(detailDTO);
            }
            PayResponseDTO payResponseDTO = cbs8Service.agentPayApply(paymentApplySubmitReqDTO,paymentApplyAgentList);
            return payResponseDTO;
        }else {
            throw new BaseSystemException("参数不能为空");
        }
    }

    /**
     * 代发代扣 结果详情查询
     *
     * @param agentPayResultRequestDTO
     * @return
     */
    @Override
    public AgentPayResultResDTO agentPayResult(AgentPayResultRequestDTO agentPayResultRequestDTO) {
        if (null != agentPayResultRequestDTO && StrUtil.isNotEmpty(agentPayResultRequestDTO.getBusNum())){
            AgentPayResultResDTO agentPayResultResDTO = cbs8Service.agentPayResult(agentPayResultRequestDTO);
            return agentPayResultResDTO;
        }
        return null;
    }

    /**
     * 保存交易明细到OA底表
     *
     * @param transactionDetailList
     */
    @Override
    public void saveTransactionDetail(List<TransactionDetailDTO> transactionDetailList) {
        if (CollectionUtils.isNotEmpty(transactionDetailList)){
            //过滤已经保存过的数据
            for (TransactionDetailDTO dto : transactionDetailList) {
                List<FormDataDTO> dataList = new ArrayList<>();
                FormDTO formDTO = new FormDTO();
                formDTO.setFormCode("formmain_0233");
                formDTO.setLoginName("yonyou");
                formDTO.setRightId("6603635988997229999.-8611088937958683581");
                String field0001=dto.getAccountNo();//我方银行账号
                String field0002=dto.getAccountName();//我方户名
                String field0003=dto.getOpenBank();//我方开户行
                String field0004=dto.getBankType();//我方银行类型
                String field0005=dto.getTransactionSerialNumber();//交易流水号
                String field0006=dto.getBankTransactionDate();//交易日期
                String field0007=dto.getBankSerialNumber();//银行流水号
                String field0008=dto.getCurrency();//币种
                String field0009=dto.getIncurredAmount();//收款金额
                String field0010=dto.getPurpose();//用途
                String field0011=dto.getDigest();//摘要
                String field0012=dto.getOppositeAccount();//对方账号
                String field0013=dto.getOppositeName();//对方户名
                String field0014=dto.getOppositeOpeningBank();//对方开户行
                String field0015=dto.getRemark();//备注
                //fields
                List<RecordFieldDTO> fields = new ArrayList<>();
                fields.add(new RecordFieldDTO("field0001",field0001,field0001));
                fields.add(new RecordFieldDTO("field0002",field0002,field0002));
                fields.add(new RecordFieldDTO("field0003",field0003,field0003));
                fields.add(new RecordFieldDTO("field0004",field0004,field0004));
                fields.add(new RecordFieldDTO("field0005",field0005,field0005));
                fields.add(new RecordFieldDTO("field0006",field0006,field0006));
                fields.add(new RecordFieldDTO("field0007",field0007,field0007));
                fields.add(new RecordFieldDTO("field0008",field0008,field0008));
                fields.add(new RecordFieldDTO("field0009",field0009,field0009));
                fields.add(new RecordFieldDTO("field0010",field0010,field0010));
                fields.add(new RecordFieldDTO("field0011",field0011,field0011));
                fields.add(new RecordFieldDTO("field0012",field0012,field0012));
                fields.add(new RecordFieldDTO("field0013",field0013,field0013));
                fields.add(new RecordFieldDTO("field0014",field0014,field0014));
                fields.add(new RecordFieldDTO("field0015",field0015,field0015));
                //masterTable
                MasterTableDTO masterTableDTO = new MasterTableDTO();
                masterTableDTO.setName("formmain_1284");
                RecordDTO recordDTO = new RecordDTO();
                recordDTO.setId(UUIDLong.longUUID());
                recordDTO.setFields(fields);
                masterTableDTO.setRecord(recordDTO);
                //dataList
                FormDataDTO formDataDTO = new FormDataDTO();
                formDataDTO.setMasterTable(masterTableDTO);
                dataList.add(formDataDTO);
                formDTO.setDataList(dataList);
            }
        }
    }

    /**
     * 无流程表单模版
     * @param type
     *  transaction 交易明细
     *  payApply 支付申请日志
     * @return
     */
    private String getXmlTemplate(String type){
        if (StrUtil.isNotEmpty(type)){
            if ("transaction".equals(type)){
                //交易明细
                return "<forms  version=\"2.1\"><formExport><summary  id=\"-6287716442449165745\" name=\"formmain_0233\"/><definitions><column  id=\"field0001\" type=\"0\" name=\"我方银行账户\" isNullable=\"false\" length=\"100\"/><column  id=\"field0002\" type=\"0\" name=\"我方户名\" isNullable=\"false\" length=\"100\"/><column  id=\"field0003\" type=\"0\" name=\"我方开户行\" isNullable=\"false\" length=\"100\"/><column  id=\"field0004\" type=\"0\" name=\"我方银行类型\" isNullable=\"false\" length=\"100\"/><column  id=\"field0005\" type=\"0\" name=\"交易流水号\" isNullable=\"false\" length=\"100\"/><column  id=\"field0006\" type=\"3\" name=\"交易日期\" isNullable=\"false\" length=\"255\"/><column  id=\"field0007\" type=\"0\" name=\"银行流水号\" isNullable=\"false\" length=\"100\"/><column  id=\"field0008\" type=\"0\" name=\"币种\" isNullable=\"false\" length=\"100\"/><column  id=\"field0009\" type=\"4\" name=\"收款金额\" isNullable=\"false\" length=\"20\"/><column  id=\"field0010\" type=\"0\" name=\"用途\" isNullable=\"false\" length=\"800\"/><column  id=\"field0011\" type=\"0\" name=\"摘要\" isNullable=\"false\" length=\"256\"/><column  id=\"field0012\" type=\"0\" name=\"对方账号\" isNullable=\"false\" length=\"100\"/><column  id=\"field0013\" type=\"0\" name=\"对方户名\" isNullable=\"false\" length=\"100\"/><column  id=\"field0014\" type=\"0\" name=\"对方开户行\" isNullable=\"false\" length=\"100\"/><column  id=\"field0015\" type=\"0\" name=\"备注\" isNullable=\"false\" length=\"800\"/></definitions><values><column  name=\"我方银行账户\"><value><![CDATA[{}]]></value></column><column  name=\"我方户名\"><value><![CDATA[{}]]></value></column><column  name=\"我方开户行\"><value><![CDATA[{}]]></value></column><column  name=\"我方银行类型\"><value><![CDATA[{}]]></value></column><column  name=\"交易流水号\"><value><![CDATA[{}]]></value></column><column  name=\"交易日期\"><value><![CDATA[{}]]></value></column><column  name=\"银行流水号\"><value><![CDATA[{}]]></value></column><column  name=\"币种\"><value><![CDATA[{}]]></value></column><column  name=\"收款金额\"><value><![CDATA[{}]]></value></column><column  name=\"用途\"><value><![CDATA[{}]]></value></column><column  name=\"摘要\"><value><![CDATA[{}]]></value></column><column  name=\"对方账号\"><value><![CDATA[{}]]></value></column><column  name=\"对方户名\"><value><![CDATA[{}]]></value></column><column  name=\"对方开户行\"><value><![CDATA[{}]]></value></column><column  name=\"备注\"><value><![CDATA[{}]]></value></column></values><subForms/></formExport></forms>";
            }else if ("payApply".equals(type)){
                //支付日志
                return "<forms  version=\"2.1\"><formExport><summary  id=\"9195604394844442459\" name=\"formmain_0232\"/><definitions><column  id=\"field0002\" type=\"0\" name=\"流程标题\" isNullable=\"false\" length=\"100\"/><column  id=\"field0003\" type=\"0\" name=\"付款主体公司\" isNullable=\"false\" length=\"100\"/><column  id=\"field0004\" type=\"0\" name=\"收款人\" isNullable=\"false\" length=\"100\"/><column  id=\"field0019\" type=\"4\" name=\"金额\" isNullable=\"false\" length=\"20\"/><column  id=\"field0005\" type=\"0\" name=\"CBS支付申请单号\" isNullable=\"false\" length=\"100\"/><column  id=\"field0006\" type=\"0\" name=\"OA单据编号\" isNullable=\"false\" length=\"100\"/><column  id=\"field0007\" type=\"0\" name=\"OA单据ID\" isNullable=\"false\" length=\"100\"/><column  id=\"field0008\" type=\"0\" name=\"OA中文表名\" isNullable=\"false\" length=\"100\"/><column  id=\"field0009\" type=\"0\" name=\"OA数据库表名\" isNullable=\"false\" length=\"100\"/><column  id=\"field0010\" type=\"0\" name=\"支付状态\" isNullable=\"false\" length=\"100\"/><column  id=\"field0011\" type=\"0\" name=\"支付信息\" isNullable=\"false\" length=\"100\"/><column  id=\"field0012\" type=\"0\" name=\"支付申请状态\" isNullable=\"false\" length=\"100\"/><column  id=\"field0014\" type=\"0\" name=\"成功标记\" isNullable=\"false\" length=\"100\"/></definitions><values><column  name=\"流程标题\"><value><![CDATA[{}]]></value></column><column  name=\"付款主体公司\"><value><![CDATA[{}]]></value></column><column  name=\"收款人\"><value><![CDATA[{}]]></value></column><column  name=\"金额\"><value><![CDATA[{}]]></value></column><column  name=\"CBS支付申请单号\"><value><![CDATA[{}]]></value></column><column  name=\"OA单据编号\"><value><![CDATA[{}]]></value></column><column  name=\"OA单据ID\"><value><![CDATA[{}]]></value></column><column  name=\"OA中文表名\"><value><![CDATA[{}]]></value></column><column  name=\"OA数据库表名\"><value><![CDATA[{}]]></value></column><column  name=\"支付状态\"><value><![CDATA[{}]]></value></column><column  name=\"支付信息\"><value><![CDATA[{}]]></value></column><column  name=\"支付申请状态\"><value><![CDATA[{}]]></value></column><column  name=\"成功标记\"><value><![CDATA[{}]]></value></column></values><subForms/></formExport></forms>";
            }
        }
        return null;
    }
}
